package net.wanji.opt.controller.induce;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.ValidationGroups;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.opt.dto.induce.GreenwaveDirDTO;
import net.wanji.opt.dto.induce.GreenwaveInducesDTO;
import net.wanji.opt.dto.induce.InduceDTO;
import net.wanji.opt.entity.DeviceInduces;
import net.wanji.opt.entity.GreenwaveInduces;
import net.wanji.opt.entity.GreenwaveInducesHist;
import net.wanji.opt.entity.InduceHist;
import net.wanji.opt.service.DeviceInducesService;
import net.wanji.opt.service.induce.GreenwaveInducesHistService;
import net.wanji.opt.service.induce.GreenwaveInducesService;
import net.wanji.opt.service.induce.InduceHistService;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotBlank;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import java.text.ParseException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * <p>
 * 绿波诱导下发状态信息
 * </p>
 *
 * @author guoliang.dong
 * @since 2024-11-22
 */
@Api(value = "GreenwaveInducesHistController", description = "绿波-绿波诱导下发状态信息")
@RestController
@RequestMapping("/greenwaveInducesHist")
@Slf4j
public class GreenwaveInducesHistController {

    @Resource
    private GreenwaveInducesHistService greenwaveInducesHistService;

    @Resource
    private GreenwaveInducesService greenwaveInducesService;

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;

    @Resource
    private DeviceInducesService deviceInducesService;

    @Resource
    private InduceHistService induceHistService;

    /**
     * 获取所有记录
     *
     * @return JsonViewObject
     */
    @ApiOperation(value = "获取所有记录", notes = "获取所有记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/byAll", produces = MediaType.APPLICATION_JSON)
    JsonViewObject getAll() {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start = System.currentTimeMillis();
        try {
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();
            List<GreenwaveInduces> greenwaveInducesList = greenwaveInducesService.list();
//            List<InduceTemplate> induceTemplateList = induceTemplateService.list();
            List<DeviceInduces> deviceInducesList = deviceInducesService.list();
            //这里从绿波表里获取基本信息
            List<GreenwaveInducesDTO> greenwaveInfoList = new ArrayList<>();
            greenwaveInfoPOList.stream().forEach(x -> {
                GreenwaveInducesDTO greenwaveInducesDTO = new GreenwaveInducesDTO();
                greenwaveInducesDTO.setGreenId(x.getId());
                greenwaveInducesDTO.setGreenName(x.getName());
                greenwaveInducesDTO.setDesignSpeed(x.getDesignSpeed());
                greenwaveInducesDTO.setStatus(x.getStatus());
                greenwaveInducesDTO.setWkt(x.getWkt());
                AtomicReference<Integer> equipCount = new AtomicReference<>(0);
                List<GreenwaveDirDTO> dirList = new ArrayList<>();
                greenwaveInducesList.stream().filter(t -> Objects.nonNull(t.getGreenId()) && t.getGreenId().equals(x.getId())).collect(Collectors.groupingBy(GreenwaveInduces::getDir)).entrySet().forEach(k -> {
                    LambdaQueryWrapper<GreenwaveInducesHist> queryWrapper = new LambdaQueryWrapper<>();
                    queryWrapper.eq(GreenwaveInducesHist::getGreenId, x.getId());
                    queryWrapper.eq(GreenwaveInducesHist::getDir, k.getKey());
                    List<GreenwaveInducesHist> newGreenwaveInducesHistList = greenwaveInducesHistService.list(queryWrapper);
                    newGreenwaveInducesHistList.stream().forEach(o -> {
                        GreenwaveDirDTO greenwaveDirDTO = new GreenwaveDirDTO();
                        greenwaveDirDTO.setDir(o.getDir());
                        greenwaveDirDTO.setStrategyId(o.getStrategyId());
                        greenwaveDirDTO.setStrategyName(o.getStrategyName());
                        greenwaveDirDTO.setMinSpeed(o.getMinSpeed());
                        greenwaveDirDTO.setMaxSpeed(o.getMaxSpeed());
                        if (Objects.nonNull(o.getControlOptTimes()) && o.getControlOptTimes().split("\\|").length > 0) {
                            try {
                                String startTime = DateUtil.format(DateUtil.parse(o.getControlOptTimes().split("\\|")[0], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), Constants.DATE_FORMAT.E_DATE_FORMAT_TIME);
                                String endTime = DateUtil.format(DateUtil.parse(o.getControlOptTimes().split("\\|")[1], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), Constants.DATE_FORMAT.E_DATE_FORMAT_TIME);
                                greenwaveDirDTO.setControlOptTimes(startTime.substring(0, startTime.lastIndexOf(":")) + "-" + endTime.substring(0, endTime.lastIndexOf(":")));
                            } catch (ParseException ex) {
                                ex.printStackTrace();
                                greenwaveDirDTO.setControlOptTimes(o.getControlOptTimes());
                            } catch (StringIndexOutOfBoundsException ex) {
                                ex.printStackTrace();
                                greenwaveDirDTO.setControlOptTimes(o.getControlOptTimes());
                            }
                        } else {
                            greenwaveDirDTO.setControlOptTimes(o.getControlOptTimes());
                        }
                        //获取诱导屏列表
                        List<InduceDTO> induceDTOList = new ArrayList<>();
                        greenwaveInducesList.stream().filter(l -> Objects.nonNull(l.getGreenId()) && l.getGreenId().equals(o.getGreenId()) && Objects.nonNull(l.getDir()) && l.getDir().equals(o.getDir())).forEach(n -> {
                            InduceDTO induceDTO = new InduceDTO();
                            induceDTO.setId(n.getId());
                            induceDTO.setEquipCode(n.getEquipCode());
                            induceDTO.setStatus(o.getStatus());
                            //获取屏幕的设备信息
                            List<DeviceInduces> deviceInduces = deviceInducesList.stream().filter(m -> Objects.nonNull(m.getEquipCode()) && m.getEquipCode().equals(n.getEquipCode())).collect(Collectors.toList());// 过滤条件// 收集结果
                            if (Objects.nonNull(deviceInduces) && deviceInduces.size() > 0) {
                                induceDTO.setEquipName(deviceInduces.get(0).getEquipName());
                                induceDTO.setRidDir(n.getDir());//
                                induceDTO.setResolutionWidth(deviceInduces.get(0).getResolutionWidth());//
                                induceDTO.setResolutionHeight(deviceInduces.get(0).getResolutionHeight());//
                                induceDTO.setWkt(deviceInduces.get(0).getWkt());
                            }
                            //关联模板信息
                            //induceDTO.setInduceTemplateList(induceTemplateList.stream().filter(x->Objects.isNull(x.getEquipCode()) || x.getEquipCode().equals(n.getEquipCode())).collect(Collectors.toList()));
                            induceDTOList.add(induceDTO);
                        });
                        greenwaveDirDTO.setInduceList(induceDTOList);
                        equipCount.updateAndGet(v -> v + induceDTOList.size());
                        dirList.add(greenwaveDirDTO);
                    });
                });
                greenwaveInducesDTO.setGreenwaveDirList(dirList);
                greenwaveInducesDTO.setEquipCount(equipCount.get());
                greenwaveInfoList.add(greenwaveInducesDTO);
            });
            jsonView.success(greenwaveInfoList);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonView;
    }


    /**
     * 根据条件查询记录
     *
     * @param greenwaveInducesHist 查询条件
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据条件查询记录", notes = "根据条件查询记录", response = JsonViewObject.class, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/byCondition", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject getByWhere(@ApiParam(value = "查询条件", required = true) @RequestBody @Validated({ValidationGroups.Query.class}) GreenwaveInducesHist greenwaveInducesHist) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start = System.currentTimeMillis();

        String jsonStr = JSON.toJSONString(greenwaveInducesHist);
        try {
            //参数校验过程中修改，兼容实现，restful入参不用map，便于进行参数逐个校验
            Map params = JSONObject.parseObject(JSONObject.toJSONString(greenwaveInducesHist), Map.class);
//            Map<String, Object> params = new HashMap<>();
//            JSONObject.parseObject(JSONObject.toJSONString(GreenwaveInducesHist), Map.class).forEach((k,v) -> params.put(StringUtils.camelToCapital(k.toString()).toLowerCase(), v));
            List<GreenwaveInducesHist> greenwaveInducesHistList = this.greenwaveInducesHistService.list();
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.findByMap(params);
            List<GreenwaveInduces> greenwaveInducesList = greenwaveInducesService.list();
//            List<InduceTemplate> induceTemplateList = induceTemplateService.list();
            List<DeviceInduces> deviceInducesList = deviceInducesService.list();
            List<GreenwaveInducesDTO> greenwaveInfoList = new ArrayList<>();
            greenwaveInfoPOList.stream().forEach(x -> {
                GreenwaveInducesDTO greenwaveInducesDTO = new GreenwaveInducesDTO();
                greenwaveInducesDTO.setGreenId(x.getId());
                greenwaveInducesDTO.setGreenName(x.getName());
                greenwaveInducesDTO.setDesignSpeed(x.getDesignSpeed());
                greenwaveInducesDTO.setWkt(x.getWkt());
                greenwaveInducesDTO.setStatus(x.getStatus());
                AtomicReference<Integer> equipCount = new AtomicReference<>(0);
                List<GreenwaveDirDTO> dirList = new ArrayList<>();
                greenwaveInducesList.stream().filter(t -> Objects.nonNull(t.getGreenId()) && t.getGreenId().equals(x.getId())).collect(Collectors.groupingBy(GreenwaveInduces::getDir)).entrySet().forEach(k -> {
                    // greenwaveInducesList.stream().collect(Collectors.toMap(p -> p.getGreenId() + "_" + p.getDir(), p -> p, (existing, replacement) -> (existing.getGreenId() + "_" + existing.getDir()).equals(replacement.getGreenId() + "_" + replacement.getDir()) ? existing : replacement)).values().stream().distinct().collect(Collectors.toList()).stream().forEach(p -> {
                    LambdaQueryWrapper<GreenwaveInducesHist> queryWrapper = new LambdaQueryWrapper<>();
                    queryWrapper.eq(GreenwaveInducesHist::getGreenId, x.getId());
                    queryWrapper.eq(GreenwaveInducesHist::getDir, k.getKey());
                    List<GreenwaveInducesHist> newGreenwaveInducesHistList = greenwaveInducesHistService.list(queryWrapper);
                    newGreenwaveInducesHistList.stream().forEach(o -> {
//                greenwaveInducesHistList.stream().filter(o -> Objects.isNull(o.getGreenId()) && Objects.equals(o.getGreenId(),x.getId())).forEach(o -> {
                        GreenwaveDirDTO greenwaveDirDTO = new GreenwaveDirDTO();
                        //这里从绿波表里获取基本信息
//                    List<GreenwaveInfoPO> greenwaveInfoByGreenId = greenwaveInfoPOList.stream().filter(n -> n.getId().equals(o.getGreenId())).collect(Collectors.toList());// 过滤条件// 收集结果
                        greenwaveDirDTO.setDir(o.getDir());
                        greenwaveDirDTO.setStrategyId(o.getStrategyId());
                        greenwaveDirDTO.setStrategyName(o.getStrategyName());
                        greenwaveDirDTO.setMinSpeed(o.getMinSpeed());
                        greenwaveDirDTO.setMaxSpeed(o.getMaxSpeed());
                        if (Objects.nonNull(o.getControlOptTimes()) && o.getControlOptTimes().split("\\|").length > 0) {
                            try {
                                String startTime = DateUtil.format(DateUtil.parse(o.getControlOptTimes().split("\\|")[0], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), Constants.DATE_FORMAT.E_DATE_FORMAT_TIME);
                                String endTime = DateUtil.format(DateUtil.parse(o.getControlOptTimes().split("\\|")[1], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), Constants.DATE_FORMAT.E_DATE_FORMAT_TIME);
                                greenwaveDirDTO.setControlOptTimes(startTime.substring(0, startTime.lastIndexOf(":")) + "-" + endTime.substring(0, endTime.lastIndexOf(":")));
                            } catch (ParseException ex) {
                                ex.printStackTrace();
                                greenwaveDirDTO.setControlOptTimes(o.getControlOptTimes());
                            } catch (StringIndexOutOfBoundsException ex) {
                                ex.printStackTrace();
                                greenwaveDirDTO.setControlOptTimes(o.getControlOptTimes());
                            }
                        } else {
                            greenwaveDirDTO.setControlOptTimes(o.getControlOptTimes());
                        }
                        //获取诱导屏列表
                        List<InduceDTO> induceDTOList = new ArrayList<>();
                        greenwaveInducesList.stream().filter(l -> Objects.nonNull(l.getGreenId()) && l.getGreenId().equals(o.getGreenId()) && Objects.nonNull(l.getDir()) && l.getDir().equals(o.getDir())).forEach(n -> {
                            InduceDTO induceDTO = new InduceDTO();
                            induceDTO.setId(n.getId());
                            induceDTO.setEquipCode(n.getEquipCode());
                            induceDTO.setStatus(o.getStatus());
                            //获取屏幕的设备信息
                            List<DeviceInduces> deviceInduces = deviceInducesList.stream().filter(m -> Objects.nonNull(m.getEquipCode()) && m.getEquipCode().equals(n.getEquipCode())).collect(Collectors.toList());// 过滤条件// 收集结果
                            if (Objects.nonNull(deviceInduces) && deviceInduces.size() > 0) {
                                induceDTO.setEquipName(deviceInduces.get(0).getEquipName());
                                induceDTO.setRidDir(n.getDir());//
                                induceDTO.setResolutionWidth(deviceInduces.get(0).getResolutionWidth());//
                                induceDTO.setResolutionHeight(deviceInduces.get(0).getResolutionHeight());//
                                induceDTO.setWkt(deviceInduces.get(0).getWkt());
                            }
                            //关联模板信息
                            //induceDTO.setInduceTemplateList(induceTemplateList.stream().filter(x -> Objects.isNull(x.getEquipCode()) || x.getEquipCode().equals(n.getEquipCode())).collect(Collectors.toList()));
                            induceDTOList.add(induceDTO);
                        });
                        greenwaveDirDTO.setInduceList(induceDTOList);
                        equipCount.updateAndGet(v -> v + induceDTOList.size());
                        dirList.add(greenwaveDirDTO);
                    });
                });
                greenwaveInducesDTO.setGreenwaveDirList(dirList);
                greenwaveInducesDTO.setEquipCount(equipCount.get());
                greenwaveInfoList.add(greenwaveInducesDTO);
            });

            List<GreenwaveInducesDTO> list = setGreenStatus(greenwaveInfoList);

            jsonView.success(list);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getByWhere error，jsonStr:{}", this.getClass().getSimpleName(), jsonStr, e);
        }
        return jsonView;
    }

    /**
     * 通过诱导屏上屏记录，修改绿波诱导发布状态
     * @param greenwaveInfoList
     * @return
     */
    private List<GreenwaveInducesDTO> setGreenStatus(List<GreenwaveInducesDTO> greenwaveInfoList) {
        LambdaQueryWrapper<InduceHist> queryWrapper = new LambdaQueryWrapper<>();
        LocalDateTime date = LocalDate.now().atStartOfDay();
        queryWrapper.ge(InduceHist::getCreateTime, date);
        List<InduceHist> list = induceHistService.list(queryWrapper);
        List<GreenwaveInducesDTO> results = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            Map<String, Integer> map = list.stream().collect(Collectors.toMap(InduceHist::getEquipCode, InduceHist::getGreenId,
                    (existing, replacement) -> existing));
            for (GreenwaveInducesDTO greenwaveInducesDTO : greenwaveInfoList) {
                Integer greenId = greenwaveInducesDTO.getGreenId();
                if (!map.containsValue(greenId)) {
                    greenwaveInducesDTO.setStatus(0);
                    results.add(greenwaveInducesDTO);
                } else {
                    List<GreenwaveDirDTO> greenwaveDirList = greenwaveInducesDTO.getGreenwaveDirList();
                    for (GreenwaveDirDTO greenwaveDirDTO : greenwaveDirList) {
                        List<InduceDTO> induceList = greenwaveDirDTO.getInduceList();
                        for (InduceDTO induceDTO : induceList) {
                            String equipCode = induceDTO.getEquipCode();
                            if (map.containsKey(equipCode)) {
                                greenwaveInducesDTO.setStatus(1);
                                results.add(greenwaveInducesDTO);
                            }
                        }
                    }
                }
            }

        } else {
            for (GreenwaveInducesDTO greenwaveInducesDTO : greenwaveInfoList) {
                greenwaveInducesDTO.setStatus(0);
                results.add(greenwaveInducesDTO);
            }
        }
        List<GreenwaveInducesDTO> collect = results.stream().distinct().collect(Collectors.toList());
        return collect;
    }

    /**
     * 根据id查询记录
     *
     * @param id
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据id查询记录", notes = "根据id查询记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON)
    JsonViewObject getById(@ApiParam(value = "记录的id", required = true, example = "1") @PathVariable("id") @NotBlank(message = "查询id不能为空") String id){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            GreenwaveInducesHist GreenwaveInducesHist = this.greenwaveInducesHistService.getById(id);
            jsonView.success(GreenwaveInducesHist);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("AbstractRestServerImpl getById error, id:{}", id, e);
        }
        return jsonView;
    }

    /**
     * 根据id删除
     *
     * @param ids
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据多个id删除记录", notes = "根据多个id删除记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "ids", dataType = "String", required = true, value = "多个记录id，用逗号分隔", example = "1,2")
    })
    @GetMapping(value = "/deleting", produces = MediaType.APPLICATION_JSON)
    JsonViewObject deleteByIds(@QueryParam("ids") @NotBlank(message = "删除ids不能为空") String ids){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        String[] idArray = ids.split(",");
        try {
            if (idArray.length > 0) {
                for (String id : idArray) {
                    jsonView = this.greenwaveInducesHistService.removeByIds(Arrays.asList(ids.split(",")))?jsonView.success():jsonView.fail();;
                }
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("DELETE_FAILED_MSG"));
            log.error("AbstractRestServerImpl deleteByIds error, id:{}", ids, e);
        }
        return jsonView;
    }

    /**
     * 新建记录
     *
     * @param greenwaveInducesHist
     * @return JsonViewObject
     */
    @ApiOperation(value = "新建记录", notes = "新建记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/creating", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject save(@ApiParam(value = "记录的JSON格式字符串", required = true) @RequestBody @Validated({ValidationGroups.Create.class}) GreenwaveInducesHist greenwaveInducesHist){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        try {
            if (greenwaveInducesHist != null) {
                jsonView = this.greenwaveInducesHistService.saveOrUpdate(greenwaveInducesHist)?jsonView.success():jsonView.fail();
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("SAVE_FAILED_MSG"));
            log.error("AbstractRestServerImpl save error, jsonStr:{}", JSON.toJSONString(greenwaveInducesHist), e);
        }
        return jsonView;
    }

    /**
     * 修改记录
     *
     * @param greenwaveInducesHist
     * @return
     */
    @ApiOperation(value = "修改记录", notes = "修改记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/updating", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject update(@ApiParam(value = "记录的JSON格式字符串", required = true) @RequestBody @Validated({ValidationGroups.Update.class}) GreenwaveInducesHist greenwaveInducesHist){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            if (greenwaveInducesHist != null) {
                jsonView = this.greenwaveInducesHistService.saveOrUpdate(greenwaveInducesHist)?jsonView.success():jsonView.fail();
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("UPDATE_FAILED_MSG"));
            log.error("AbstractRestServerImpl update error, jsonStr:{}", JSON.toJSONString(greenwaveInducesHist), e);
        }
        return jsonView;
    }
}
